function drawFloor()
	--drawSky()
	drawCeiling()
	
	-- Transform sprite with the inverse camera matrix
	local invDet = 1.0 / (player.planeX * player.dirY - player.dirX * player.planeY);

	local addX = 1.0
	local addSmall = 0.5

	-- NOTE: Add math.abs to the Xs to do "paper-folding" effect
	for i,v in ipairs(floorTable) do
		if v.x > 0 and v.x <= #floor[1] and v.y > 0 and v.y <= #floor then
			local v1, v2, v3, v4, canDraw = floorTile(v.x, v.y, addX, invDet)
			local transformY = invDet * (-player.planeY * (v.x - player.posX) + player.planeX * (v.y - player.posY));

			local deathColor = (transformY / maxFogDist)
			if deathColor < 0 then deathColor = 0 end
			if deathColor > 1 then deathColor = 1 end
			local colorr = {0, 0, 0, 255 * deathColor}

			if canDraw then
				love.graphics.setColor(colorr)
				pers.quad(img[floor[v.x][v.y]], v1, v2, v3, v4)
				love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
			else
				local v1, v2, v3, v4, canDraw = floorTile(v.x, v.y, addSmall, invDet)
				if canDraw then
					love.graphics.setColor(colorr)
					pers.quad(cornerImg[floor[v.x][v.y]][1], v1, v2, v3, v4)
					love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
				end

				local v1, v2, v3, v4, canDraw = floorTile(v.x + addSmall, v.y, addSmall, invDet)
				if canDraw then
					love.graphics.setColor(colorr)
					pers.quad(cornerImg[floor[v.x][v.y]][2], v1, v2, v3, v4)
					love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
				end

				local v1, v2, v3, v4, canDraw = floorTile(v.x, v.y + addSmall, addSmall, invDet)
				if canDraw then
					love.graphics.setColor(colorr)
					pers.quad(cornerImg[floor[v.x][v.y]][3], v1, v2, v3, v4)
					love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
				end

				local v1, v2, v3, v4, canDraw = floorTile(v.x + addSmall, v.y + addSmall, addSmall, invDet)
				if canDraw then
					love.graphics.setColor(colorr)
					pers.quad(cornerImg[floor[v.x][v.y]][4], v1, v2, v3, v4)
					love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
				end
			end
		end
	end
end

function drawCeiling()
	-- Transform sprite with the inverse camera matrix
	local invDet = 1.0 / (player.planeX * player.dirY - player.dirX * player.planeY);

	local addX = 1.0
	local addSmall = 0.5
	local level = numLevels
	local addToSprites = false

	-- NOTE: Add math.abs to the Xs to do "paper-folding" effect
	for i,v in ipairs(ceilingTable) do
		if v.x > 0 and v.x <= #ceiling[1] and v.y > 0 and v.y <= #ceiling then
			if map[v.x][v.y] == -7 or map[v.x][v.y] == -8 then
				level = 1
				addToSprites = true
			else
				level = numLevels
				addToSprites = false
			end

			local v1, v2, v3, v4, canDraw = ceilingTile(v.x, v.y, addX, invDet, level)
			local transformY = invDet * (-player.planeY * (v.x + 0.5 - player.posX) + player.planeX * (v.y + 0.5 - player.posY));

			local deathColor = (transformY / maxFogDist)
			if deathColor < 0 then deathColor = 0 end
			if deathColor > 1 then deathColor = 1 end
			local colorr = {0, 0, 0, 255 * deathColor}

			if canDraw then
				if not addToSprites then
					love.graphics.setColor(colorr)
					pers.quad(img[ceiling[v.x][v.y]], v1, v2, v3, v4)
					love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
				else
					local i = {
						id = ceiling[v.x][v.y],
						v1 = v1,
						v2 = v2,
						v3 = v3,
						v4 = v4,
						cov = {9001, 9001},
						type = "ceil",
						color = colorr,
						z = math.sqrt((player.posX - v.x - 0.5)^2 + (player.posY - v.y - 0.5)^2) + 0.5,
					}

					local tranY = invDet * (-player.planeY * (v.x - player.posX) + player.planeX * (v.y - player.posY))
					if map[v.x][v.y] == -7 then
						if player.posX >= v.x + 0.5 then
							i.cov[1] = v2[1] + 1
							i.cov[2] = v3[1] + 1

							tranY = invDet * (-player.planeY * (v.x + 1 - player.posX) + player.planeX * (v.y + 1 - player.posY))
						else
							i.cov[1] = v4[1]
							i.cov[2] = v1[1] + 1

							tranY = invDet * (-player.planeY * (v.x - player.posX) + player.planeX * (v.y - player.posY))
						end
					elseif map[v.x][v.y] == -8 then
						if player.posY >= v.y + 0.5 then
							i.cov[1] = v4[1]
							i.cov[2] = v3[1] + 1

							tranY = invDet * (-player.planeY * (v.x + 1 - player.posX) + player.planeX * (v.y + 1 - player.posY))
						else
							i.cov[1] = v1[1]
							i.cov[2] = v2[1] + 1

							tranY = invDet * (-player.planeY * (v.x - player.posX) + player.planeX * (v.y - player.posY))
						end
					end

					if i.cov[2] < i.cov[1] then local temp = i.cov[1]; i.cov[1] = i.cov[2]; i.cov[2] = temp end
					for stripe = math.floor(i.cov[1]), i.cov[2] do
						if  (tranY > 0 and stripe >= 0 and stripe <= w and tranY < zBuffer[stripe]) then
							if stripe > i.cov[1] then
								i.cov[1] = stripe
							end
							if stripe < i.cov[2] then
								i.cov[2] = stripe
							end
						end
					end
					i.cov[1] = i.cov[1] + 1

					table.insert(spritePool, i)
				end
			else
				local v1, v2, v3, v4, canDraw = ceilingTile(v.x, v.y, addSmall, invDet, level)
				if canDraw then
					if not addToSprites then
						love.graphics.setColor(colorr)
						pers.quad(cornerImg[ceiling[v.x][v.y]][1], v1, v2, v3, v4)
						love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
					else
						local i = {
							id = ceiling[v.x][v.y],
							subID = 1,
							v1 = v1,
							v2 = v2,
							v3 = v3,
							v4 = v4,
							type = "ceil2",
							color = colorr,
							z = math.sqrt((player.posX - v.x - 0.25)^2 + (player.posY - v.y - 0.25)^2) + 0.5,
						}
						table.insert(spritePool, i)
					end
				end

				local v1, v2, v3, v4, canDraw = ceilingTile(v.x + addSmall, v.y, addSmall, invDet, level)
				if canDraw then
					if not addToSprites then
						love.graphics.setColor(colorr)
						pers.quad(cornerImg[ceiling[v.x][v.y]][2], v1, v2, v3, v4)
						love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
					else
						local i = {
							id = ceiling[v.x][v.y],
							subID = 2,
							v1 = v1,
							v2 = v2,
							v3 = v3,
							v4 = v4,
							type = "ceil2",
							color = colorr,
							z = math.sqrt((player.posX - v.x - addSmall - 0.25)^2 + (player.posY - v.y - 0.25)^2) + 0.5,
						}
						table.insert(spritePool, i)
					end
				end

				local v1, v2, v3, v4, canDraw = ceilingTile(v.x, v.y + addSmall, addSmall, invDet, level)
				if canDraw then
					if not addToSprites then
						love.graphics.setColor(colorr)
						pers.quad(cornerImg[ceiling[v.x][v.y]][3], v1, v2, v3, v4)
						love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
					else
						local i = {
							id = ceiling[v.x][v.y],
							subID = 3,
							v1 = v1,
							v2 = v2,
							v3 = v3,
							v4 = v4,
							type = "ceil2",
							color = colorr,
							z = math.sqrt((player.posX - v.x - 0.25)^2 + (player.posY - v.y - addSmall - 0.25)^2) + 0.5,
						}
						table.insert(spritePool, i)
					end
				end

				local v1, v2, v3, v4, canDraw = ceilingTile(v.x + addSmall, v.y + addSmall, addSmall, invDet, level)
				if canDraw then
					if not addToSprites then
						love.graphics.setColor(colorr)
						pers.quad(cornerImg[ceiling[v.x][v.y]][4], v1, v2, v3, v4)
						love.graphics.polygon("fill", {v1[1], v1[2], v2[1], v2[2],v3[1], v3[2],v4[1], v4[2]})
					else
						local i = {
							id = ceiling[v.x][v.y],
							subID = 4,
							v1 = v1,
							v2 = v2,
							v3 = v3,
							v4 = v4,
							type = "ceil2",
							color = colorr,
							z = math.sqrt((player.posX - v.x - addSmall - 0.25)^2 + (player.posY - addSmall - v.y - 0.25)^2) + 0.5,
						}
						table.insert(spritePool, i)
					end
				end
			end
		end
	end
end

function drawSky()
	local pi = math.pi
    local ss = 1 / pixelScale
	local sx = -((skyImg:getWidth() / (pi * 2)) * (getPlayerRot() % (pi * 2)))
	local sy = -player.tLim - (player.tLim - player.tilt)

	for i = -1, 1 do
	    love.graphics.setColor(255, 255, 255)
		love.graphics.draw(skyImg, sx + (skyImg:getWidth() * i), sy, 0, 1 / pixelScale, 1 / pixelScale)
	end
end