-- Spawns a new sprite
function newSprite(x, y, textureID)
	table.insert(sprites, {x = x, y = y, id = textureID})
end

-- Spawns a new door
function newDoor(x, y, textureID, wide, mapX, mapY) -- Wide determines whether or not the door is horizontal or not
	table.insert(doors, {x = x, y = y, id = textureID, wide = wide, open = false, opening = false, mapX = mapX, mapY = mapY, movementLeft = 1,})
end

-- NPC Spawn
function spawnNPC(x, y, r) -- For the R, use degrees. Adding is turning left, subtracting in turning right
	table.insert(sprites, {x = x, y = y, r = r, ttype = "npc", id = 1, curImg = 1})
end

function npcAngle()
	for i,v in ipairs(sprites) do
		if v.ttype == "npc" then
			local v1 = {x=math.cos(v.r), y=math.sin(v.r)}
			local b = math.atan2(player.posY - v.y, player.posX - v.x)
			local v2 = {x=math.cos(b), y=math.sin(b)}
			local a = math.deg(math.acos((v1.x*v2.x + v1.y*v2.y) / (((v1.x^2 + v1.y^2)^0.5) * ((v2.x^2 + v2.y^2)^0.5))))
			if angleDifference(b, v.r) < 0 then a = -a end

			if a >= 0 and a <= 22.5 then
				v.curImg = 1
			elseif a > 22.5 and a <= 67.5 then
				v.curImg = 5
			elseif a > 67.5 and a <= 112.5 then
				v.curImg = 3
			elseif a > 112.5 and a <= 157.5 then
				v.curImg = 6
			elseif a > 157.5 and a <= 180 then
				v.curImg = 2

			elseif a >= -180 and a <= -157.5 then
				v.curImg = 2
			elseif a > -157.5 and a <= -112.5 then
				v.curImg = 8
			elseif a > -112.5 and a <= -67.5 then
				v.curImg = 4
			elseif a > -67.5 and a <= -22.5 then
				v.curImg = 7
			elseif a > -22.5 and a <= 0 then
				v.curImg = 1
			end
		end
	end
end

function angleDifference(t1, t2)
   return (t1 - t2 + math.pi) % (math.pi * 2) - math.pi
end

-- Checks for a door
function checkDoor(mapX, mapY)
	for i=1, #doors do
		if doors[i].mapX == mapX and doors[i].mapY == mapY then
			if doors[i].open then
				return true
			else
				doors[i].opening = true
				return false
			end
		end
	end

	return false
end

-- Updates Doors
function updateDoor(dt)
	for i=1, #doors do
		if doors[i].opening then
			if doors[i].wide then
				doors[i].y = doors[i].y - (dt / 2)
			else
				doors[i].x = doors[i].x - (dt / 2)
			end
			doors[i].movementLeft = doors[i].movementLeft - (dt / 2)

			if doors[i].movementLeft <= 0 then
				doors[i].opening = false
				doors[i].open = true
			end
		end
	end
end

-- Gets the rotation of the player
function getPlayerRot()
	return math.atan2(player.dirY, player.dirX)
end

-- Sort Algorithm
function zSort(object1, object2)
    return object1.z > object2.z
end

-- Check Match
function checkMatch(t1, t2)
	local match = false
	for i,v in ipairs(t1) do
		if v.x == t2.x and v.y == t2.y then
			match = true
		end
	end

	return match
end

-- Floor Tile
function floorTile(x, y, additive, invDet)
	local allowedDraw = true
	local spriteX = x - player.posX;
	local spriteY = y - player.posY;

	tx = invDet * (player.dirY * (spriteX) - player.dirX * (spriteY));
	ty = invDet * (-player.planeY * (spriteX) + player.planeX * (spriteY));
	if ty <= 0 then allowedDraw = false end

	local X1 = ((w / 2) * (1 + tx / ty));
	local Y1 = ((h - player.zHeight) / ty) / 2 + ((h / 2) + player.tilt)

	tx = invDet * (player.dirY * (spriteX + additive) - player.dirX * (spriteY));
	ty = invDet * (-player.planeY * (spriteX + additive) + player.planeX * (spriteY));
	if ty <= 0 then allowedDraw = false end

	local X2 = ((w / 2) * (1 + tx / ty));
	local Y2 = ((h - player.zHeight) / ty) / 2 + ((h / 2) + player.tilt)

	tx = invDet * (player.dirY * (spriteX + additive) - player.dirX * (spriteY + additive));
	ty = invDet * (-player.planeY * (spriteX + additive) + player.planeX * (spriteY + additive));
	if ty <= 0 then allowedDraw = false end

	local X3 = ((w / 2) * (1 + tx / ty));
	local Y3 = ((h - player.zHeight) / ty) / 2 + ((h / 2) + player.tilt)

	tx = invDet * (player.dirY * (spriteX) - player.dirX * (spriteY + additive));
	ty = invDet * (-player.planeY * (spriteX) + player.planeX * (spriteY + additive));
	if ty <= 0 then allowedDraw = false end

	local X4 = ((w / 2) * (1 + tx / ty));
	local Y4 = ((h - player.zHeight) / ty) / 2 + ((h / 2) + player.tilt)

	return {X1, Y1}, {X2, Y2}, {X3, Y3}, {X4, Y4}, allowedDraw
end

-- Ceiling Tile
function ceilingTile(x, y, additive, invDet, level)
	local tempLevel = numLevels
	local allowedDraw = true
	local spriteX = x - player.posX;
	local spriteY = y - player.posY;

	if level ~= numLevels then numLevels = level end

	tx = invDet * (player.dirY * (spriteX) - player.dirX * (spriteY));
	ty = invDet * (-player.planeY * (spriteX) + player.planeX * (spriteY));
	if ty <= 0 then allowedDraw = false end

	local X1 = ((w / 2) * (1 + tx / ty));
	local Y1 = -((h * (getOne() + (1 / (2^(numLevels-1)))) + player.zHeight * ((1 / (2^(numLevels-1))))) / ty) * (0.5 * (2^(numLevels-1))) + ((h / 2) + player.tilt)

	tx = invDet * (player.dirY * (spriteX + additive) - player.dirX * (spriteY));
	ty = invDet * (-player.planeY * (spriteX + additive) + player.planeX * (spriteY));
	if ty <= 0 then allowedDraw = false end

	local X2 = ((w / 2) * (1 + tx / ty));
	local Y2 = -((h * (getOne() + (1 / (2^(numLevels-1)))) + player.zHeight * ((1 / (2^(numLevels-1))))) / ty) * (0.5 * (2^(numLevels-1))) + ((h / 2) + player.tilt)

	tx = invDet * (player.dirY * (spriteX + additive) - player.dirX * (spriteY + additive));
	ty = invDet * (-player.planeY * (spriteX + additive) + player.planeX * (spriteY + additive));
	if ty <= 0 then allowedDraw = false end

	local X3 = ((w / 2) * (1 + tx / ty));
	local Y3 = -((h * (getOne() + (1 / (2^(numLevels-1)))) + player.zHeight * ((1 / (2^(numLevels-1))))) / ty) * (0.5 * (2^(numLevels-1))) + ((h / 2) + player.tilt)

	tx = invDet * (player.dirY * (spriteX) - player.dirX * (spriteY + additive));
	ty = invDet * (-player.planeY * (spriteX) + player.planeX * (spriteY + additive));
	if ty <= 0 then allowedDraw = false end

	local X4 = ((w / 2) * (1 + tx / ty));
	local Y4 = -((h * (getOne() + (1 / (2^(numLevels-1)))) + player.zHeight * ((1 / (2^(numLevels-1))))) / ty) * (0.5 * (2^(numLevels-1))) + ((h / 2) + player.tilt)

	numLevels = tempLevel
	return {X1, Y1}, {X2, Y2}, {X3, Y3}, {X4, Y4}, allowedDraw
end

-- Function because I couldn't be bothered to figure out the math befind it
function getOne()
	if numLevels > 1 then return 1
	else return 0 end
end

function math.sign(n) return n>0 and 1 or n<0 and -1 or 0 end

-- Player Input
function input(dt)
	local moveSpeed, strafeSpeed = dt * player.moveMod, dt * player.moveMod
	local rotSpeed = dt * player.rotMod
	local tiltSpeed = dt * player.tiltMod
	local z = 0
	local t1 = player.tLimTop
	local t2 = player.tLimDown

	if love.keyboard.isDown("w") then
		local mapX = math.floor(player.posX + player.dirX * moveSpeed + player.dirX * z)
		local mapY = math.floor(player.posY + player.dirY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8) and checkDoor(mapX, mapY))) then
			player.posX = player.posX + player.dirX * moveSpeed
		end

		local mapX = math.floor(player.posX + player.dirX * z)
		local mapY = math.floor(player.posY + player.dirY * moveSpeed + player.dirY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY + player.dirY * moveSpeed
		end
	end
	if love.keyboard.isDown("s") then
		local mapX = math.floor(player.posX - player.dirX * moveSpeed - player.dirX * z)
		local mapY = math.floor(player.posY - player.dirY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posX = player.posX - player.dirX * moveSpeed
		end

		local mapX = math.floor(player.posX - player.dirX * z)
		local mapY = math.floor(player.posY - player.dirY * moveSpeed - player.dirY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY - player.dirY * moveSpeed
		end
	end
	if love.keyboard.isDown("d") then
		local mapX = math.floor(player.posX + player.planeX * moveSpeed + player.planeX * z)
		local mapY = math.floor(player.posY + player.planeY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posX = player.posX + player.planeX * strafeSpeed
		end

		local mapX = math.floor(player.posX + player.planeX * z)
		local mapY = math.floor(player.posY + player.planeY * moveSpeed + player.planeY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY + player.planeY * strafeSpeed
		end
	end
	if love.keyboard.isDown("a") then
		local mapX = math.floor(player.posX - player.planeX * moveSpeed - player.planeX * z)
		local mapY = math.floor(player.posY - player.planeY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posX = player.posX - player.planeX * strafeSpeed
		end

		local mapX = math.floor(player.posX - player.planeX * z)
		local mapY = math.floor(player.posY - player.planeY * moveSpeed - player.planeY * z)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY - player.planeY * strafeSpeed
		end 
	end

	player.zHeight = player.zHeight + player.vy
	if player.zHeight >= 0 then
		player.vy = 0
		player.zHeight = 0
	else
		player.vy = player.vy + 400*dt
	end

	if love.keyboard.isDown("lshift") then
		player.zHeight = player.zHeight + player.crouchHeight

		player.moveMod = player.movement / 2
	else
		player.moveMod = player.movement
	end

	if grabbed then
		if player.dx ~= 0 then
			local turnSpeed = rotSpeed * player.dx

			local oldDirX = player.dirX
			player.dirX = player.dirX * math.cos(turnSpeed) - player.dirY * math.sin(turnSpeed)
			player.dirY = oldDirX * math.sin(turnSpeed) + player.dirY * math.cos(turnSpeed)
			local oldPlaneX = player.planeX
			player.planeX = player.planeX * math.cos(turnSpeed) - player.planeY * math.sin(turnSpeed)
			player.planeY = oldPlaneX * math.sin(turnSpeed) + player.planeY * math.cos(turnSpeed)
		end
		
		-- Head Tilt (Working)
		if player.dy ~= 0 then
			local ttiltSpeed = tiltSpeed * player.dy

			if player.tilt + ttiltSpeed > -t2 and player.tilt + ttiltSpeed < t1 then
				player.tilt = player.tilt + ttiltSpeed
			end
		end

		player.dx = 0; player.dy = 0
	end

	--if player.posY < 10 then player.zHeight = -2*h end
end

-- Corner images for the floor
function genCorner()
	for i = 1, #img do
		local fullImg = love.image.newImageData(directory[i])
		local widthh, heightt = fullImg:getDimensions()
		local fakeX, fakeY = 0, 0
		cornerImg[i] = {}

		local imageData = love.image.newImageData(widthh / 2, heightt / 2)
		for x = 0, (widthh / 2) - 1 do
			for y = 0, (heightt / 2) - 1 do
				local r, g, b, a = fullImg:getPixel(x, y)
				imageData:setPixel(fakeX, fakeY, r, g, b, a)

				fakeY = fakeY + 1
			end
			fakeX = fakeX + 1
			fakeY = 0
		end
		fakeX, fakeY = 0, 0

		cornerImg[i][1] = love.graphics.newImage(imageData)

		for x = (widthh / 2), widthh - 1 do
			for y = 0, (heightt / 2) - 1 do
				local r, g, b, a = fullImg:getPixel(x, y)
				imageData:setPixel(fakeX, fakeY, r, g, b, a)

				fakeY = fakeY + 1
			end
			fakeX = fakeX + 1
			fakeY = 0
		end
		fakeX, fakeY = 0, 0

		cornerImg[i][2] = love.graphics.newImage(imageData)

		for x = 0, (widthh / 2) - 1 do
			for y = (heightt / 2), heightt - 1 do
				local r, g, b, a = fullImg:getPixel(x, y)
				imageData:setPixel(fakeX, fakeY, r, g, b, a)

				fakeY = fakeY + 1
			end
			fakeX = fakeX + 1
			fakeY = 0
		end
		fakeX, fakeY = 0, 0

		cornerImg[i][3] = love.graphics.newImage(imageData)

		for x = (widthh / 2), widthh - 1 do
			for y = (heightt / 2), heightt - 1 do
				local r, g, b, a = fullImg:getPixel(x, y)
				imageData:setPixel(fakeX, fakeY, r, g, b, a)

				fakeY = fakeY + 1
			end
			fakeX = fakeX + 1
			fakeY = 0
		end

		cornerImg[i][4] = love.graphics.newImage(imageData)
	end
end