love.graphics.setDefaultFilter("nearest", "nearest")

-- Loads everything
require("cde/load")

-- Misc. functions
require("cde/misc")

-- Floor related functions
require("cde/floor")

-- The majority of raycasting-related functions
require("cde/raycasting")

-- Spritecasting
require("cde/spritecasting")

-- Table Serialiser
require("cde/tableSer")

-- Shader
pers = require("cde/pers")

local timerr = 0

function love.load()
	loadEverything()
end

function love.update(dt)
	-- Key Input
	input(dt)

	for x = 0, w, 1 do
		castRay(x)
	end

	sprites[#sprites-1].r = sprites[#sprites-1].r + math.rad(30) * dt
	sprites[#sprites].r = sprites[#sprites].r - math.rad(30) * dt

	updateDoor(dt)
	castSprites()
	castDoors()

	if not curShrink then
		curSize = curSize + dt * 50

		if curSize > maxSize then
			curShrink = true
		end
	else
		curSize = curSize - dt * 50

		if curSize < minSize then
			curShrink = false
		end
	end
end

function love.draw()
	drawFloor()
	
	-- Sort the spritePool based on the screen depth
	table.sort(spritePool, zSort)
	npcAngle()

	-- Walls draw before the sprites
	for i,v in ipairs(wallPool) do
		love.graphics.setColor(v.color)
		if img[v.texture] then
			for k=1,numLevels do
				love.graphics.draw(img[v.texture], quads[v.hor], v.x, v.start - (k-1) * (textureHeight * v.scale), 0, 1, v.scale)
			end
		end
	end

	for i,v in ipairs(spritePool) do
		love.graphics.setColor(v.color)
		if v.type == "wall" then
			if img[v.texture] then
				for k=1,numLevels do
					if v.onlyTop then
						if k ~= 1 then
							love.graphics.draw(img[v.texture], quads[v.hor], v.x, v.start - (k-1) * (textureHeight * v.scale), 0, 1, v.scale)
						end
					else
						love.graphics.draw(img[v.texture], quads[v.hor], v.x, v.start - (k-1) * (textureHeight * v.scale), 0, 1, v.scale)
					end
				end
			end
		elseif v.type == "sprite" then
			if v.ttype == "sprite" then
				love.graphics.draw(spriteImg[v.id], quads[v.hor], v.x, v.y, 0, 1, v.scale)
			elseif v.ttype == "npc" then
				if npcImg[v.id][v.curImg] then
					love.graphics.draw(npcImg[v.id][v.curImg], quads[v.hor], v.x, v.y, 0, 1, v.scale)
				end
			end
		elseif v.type == "door" then
			local beginSt = v.cov[1]
			local endSt = v.cov[2]
			if beginSt > endSt then beginSt = v.cov[2]; endSt = v.cov[1] end

			love.graphics.setScissor(beginSt, 0, endSt - beginSt, h)
				love.graphics.setColor(v.color)
				pers.quad(img[v.id], v.v1, v.v2, v.v3, v.v4)
				love.graphics.polygon("fill", {v.v1[1], v.v1[2], v.v2[1], v.v2[2],v.v3[1], v.v3[2],v.v4[1], v.v4[2],})
			love.graphics.setScissor()
		elseif v.type == "ceil" then
			local beginSt = v.cov[1]
			local endSt = v.cov[2]
			if beginSt > endSt then beginSt = v.cov[2]; endSt = v.cov[1] end

			love.graphics.setScissor(beginSt, 0, endSt - beginSt, h)
				love.graphics.setColor(v.color)
				pers.quad(img[v.id], v.v1, v.v2, v.v3, v.v4)
				love.graphics.polygon("fill", {v.v1[1], v.v1[2], v.v2[1], v.v2[2],v.v3[1], v.v3[2],v.v4[1], v.v4[2],})
			love.graphics.setScissor()
		elseif v.type == "ceil2" then
			love.graphics.setColor(v.color)
			pers.quad(cornerImg[v.id][v.subID], v.v1, v.v2, v.v3, v.v4)
			love.graphics.polygon("fill", {v.v1[1], v.v1[2], v.v2[1], v.v2[2],v.v3[1], v.v3[2],v.v4[1], v.v4[2],})
		end
	end

	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(crosshair, w/2, h/2, 0, 1, 1, crosshair:getWidth()/2, crosshair:getHeight()/2)
		
	-- Empty the pools
	spritePool = {}
	wallPool = {}
	floorTable = {}
	ceilingTable = {}

	love.graphics.setColor(255, 60, 60)
	love.graphics.print(string.format("FPS: %1.f\nPlayer Rotation: %1.f\nX: %.1f\nY: %.1f\nzHeight: %1.f", love.timer.getFPS(), math.deg(getPlayerRot()), player.posX, player.posY, player.zHeight))
end

function love.keypressed(key)
	if key == "escape" then love.event.quit() end
	if key == "tab" then
		grabbed = not grabbed
		love.mouse.setRelativeMode(grabbed)
	end

	if key == " " and player.zHeight == 0 then
		player.vy = -120
	end
end

function love.mousemoved(x, y, dx, dy)
	if grabbed then
		player.dx = -dx
		player.dy = -dy
	end
end